#ifndef MAINWINDOW_H 
#define MAINWINDOW_H

#include <qpopupmenu.h>
#include <qmainwindow.h>
#include <qtextedit.h>
#include <qintdict.h>
#include <qcanvas.h>
#include <qlineedit.h>
#include <qaction.h>
#include <qcursor.h>
#include <qtabwidget.h>
#include <qmap.h>

#include "xml.h"
#include "texteditor.h"
#include "mapeditor.h"
#include "findwindow.h"

class Main : public QMainWindow 
{
    Q_OBJECT

public:
    Main(QWidget* parent=0, const char* name=0, WFlags f=0);
    ~Main();
	void loadCmdLine();
	void statusMessage (const QString &);

public slots:
    void fileNew();

protected:
	void closeEvent( QCloseEvent* );

private:
    void setupFileActions();
    void setupEditActions();
    void setupFormatActions();
    void setupViewActions();
    void setupWindowActions();
    void setupFlagActions();
    void setupSettingsActions();
    void setupTestActions();
    void setupHelpActions();
    void setupContextMenus();
	void setupLastMapsMenu();
	void hideEvent (QHideEvent * );
	void showEvent (QShowEvent * );
	bool reallyWriteDirectory(const QString&);
	QString browseDirectory(const QString&);
	MapEditor* currentMapEditor() const;
    
private slots:
    void newView();
	void editorChanged(QWidget*);

    void fileLoad(QString ,const LoadMode &);
    void fileLoad(const LoadMode &);
    void fileLoad();
	void fileLoadLast(int);
    void fileSave(const SaveMode & );
    void fileSave();
    void fileSaveAs(const SaveMode &);
    void fileSaveAs();
    void fileImportDir();
    void fileExportXML();
    void fileExportHTML();
    void fileExportXHTML();
    void fileExportImage(int);
    void fileExportASCII();
    void fileCloseMap();
    void filePrint();
    void fileExitVYM();

    void editUndo();	
    void editRedo();	
    void editCopy();	
    void editPaste();	
    void editCut();	
    void editOpenFindWindow();
	void editFind(QString);
	void editFindChanged();
public slots:
	void editOpenURL();
private slots:
	void editURL();
	void editHeading2URL();
	void editBugzilla2URL();
	void editVymLink();
public slots:
	void editOpenVymLink();
private slots:
	void editDeleteVymLink();
	void editMapInfo();
    void editMoveUp();	
    void editMoveDown();	
    void editToggleScroll();
    void editUnScrollAll();
    void editHeading();
    void editNewBranch();
    void editNewBranchAbove();
    void editNewBranchBelow();
    void editImportAdd();
    void editImportReplace();
    void editSaveBranch();
    void editDeleteSelection();
    void editUpperBranch();
    void editLowerBranch();
    void editLeftBranch();
    void editRightBranch();
    void editFirstBranch();
    void editLastBranch();
    void editLoadImage();
    void editSaveImage(int);
    void editToggleFloatExport();

    void formatSelectColor();
    void formatPickColor();
    void colorChanged(QColor);
    void formatColorItem();
    void formatColorBranch();
	void formatLinkStyleLine();
	void formatLinkStyleParabel();
	void formatLinkStylePolyLine();
	void formatLinkStylePolyParabel();
    void formatSelectBackColor();
    void formatSelectLinkColor();
    void formatToggleLinkColorHint();
	void formatFrameNone();
	void formatFrameRectangle();

    void viewZoomReset();
    void viewZoomIn();
    void viewZoomOut();

public slots:
	bool settingsPDF();
	bool settingsURL();

	void windowToggleNoteEditor();
private slots:
	void windowNextEditor();
	void windowPreviousEditor();
	void windowShowNoteEditor();
	void windowHideNoteEditor();

	void standardFlagChanged();

    void testFunction();
    void testShowClipboard();

    void helpDoc();
    void helpAbout();
    void helpAboutQT();

private:
    QCanvas* canvas;
	QTabWidget *tabWidget;
	FindWindow *findWindow;
	QStringList lastMaps;
	int maxLastMaps;
	QString lastFileDir;
};


#endif

